// Debug-Modul

#include "Debug.h"

void AdrDump(void) {
  // Ausgabe der Adressen der "Magnetartikel"
  uint16_t Adr = 0;

  Serial.println();
  Serial.print(F("Digital-Adressen:"));

  for (int y = 0; y < YDIM; y++) {
    Serial.println();
    Serial.print(F("Y= ")); Serial.print(y); Serial.print(F(":"));

    for (int x = 0; x < XDIM + 1; x++) {
      Adr = StpA[y][x] & ADRMASK;
      if (Adr) {
        Serial.print(F(" x= ")); Serial.print(x); Serial.print(F(": "));
        Serial.print(Adr); Serial.print(',');
        Adr =  StpA[y][x]; Serial.print(F(" 0x")); Serial.print(Adr, HEX);
      }
    } // for x
  } // for y
  Serial.println();
} // AdrDump()



void CheckRMArray(void) {
  Serial.println(F("Belegtmelder und Fstr.Aufloesung Koord.:"));
  for (int k = 0; k < RMDIM; k++) {
    Serial.print(F("i: ")); if (k < 10) Serial.print(' '); Serial.print(k); Serial.print(' ');

    Serial.print(RMKoord[k].x); Serial.print(F(",")); Serial.print(RMKoord[k].y); Serial.print(' ');
    Serial.print(RMKoord[k].Rx); Serial.print(F(",")); Serial.print(RMKoord[k].Ry);
    Serial.println();
  }
  Serial.print(F("Anzahl RM: ")); Serial.println(RMmax);
} // CheckRMArray()



void BMdump(void) {
  Serial.println(F("Belegtmelder Dump:"));
  for (int k = 0; k < RMDIM; k++) {
    if ( k < 10)   Serial.print(' '); Serial.print(k); Serial.print(' ');
    if ((k + 1) < 10)Serial.print(' '); Serial.print(k + 1); Serial.print(' ');

    uint8_t xko = RMKoord[k].x;
    uint8_t yko = RMKoord[k].y;
    Serial.print(xko); Serial.print('/');
    Serial.print(yko); Serial.print(' ');
    Serial.print(RMKoord[k].RevRel); Serial.print(F(" Ele: "));
    Serial.print(StpM[yko][xko]);
    Serial.println();
  }
} // BMdump()



void BMZustandDump(void) {
  Serial.println(); Serial.print(F("123456789_123456789_"));
  Serial.println(F("1234"));
  for (int k = 0; k < RMDIM; k++) {
    if (RMKoord[k].y + RMKoord[k].x) {
      if (StpM[RMKoord[k].y][RMKoord[k].x] & DETECTSET) Serial.print('B');
      else Serial.print('c');
    }
    else Serial.print(' ');
  } // for
  Serial.println();
} // BMZustandDump()


void SigDump(void) {
  // Ausgabe aller Signale mit und ohne Schaltadressen
  Serial.println(F("Signal Dump:"));
  uint8_t Sig = 0;
  for (int x = 0; x < XDIM + 1; x++) {
    for (int y = 0; y < YDIM; y++) {
      Sig = StpS[y][x] & 0x1F;
      if (Sig) {
        Serial.print(Sig);
        Serial.print(' '); Serial.print(x);
        Serial.print(','); Serial.println(y);
      }
    }
  }
} // SigDump()



void FWegDump(void) { // Anzeige der Fweg Nummer in Bildschirmecke unten Links: x/y 000

  Serial.println(F("Fahrwege:"));
  for (int wegnum = 0; wegnum < GL_AnzFweg; wegnum++) {
    if (wegnum < 10) Serial.print(' '); Serial.print(wegnum); Serial.print(' ');
    for (int wlage = 0; wlage < XWnum; wlage++) {
      if (Fweg[wegnum][wlage]) {
        Serial.print(Fweg[wegnum][wlage]); Serial.print(' ');
      }
    }
    Serial.println();
  }

} // FWegDump()


void Tracker(uint8_t x, uint8_t y, uint8_t Ele) { // für ein Element kein, CR, fortlaufender Aufruf

  uint8_t Gb_Ele = Ele & 0x1F;

  if (GL_DebOpt & 0b00010000)  // 16
  {
    Serial.print(x); Serial.print('/'); Serial.print(y);
    Serial.print(F(":")); if ((Gb_Ele) < 8) Serial.print(' '); ; Serial.print(Gb_Ele);
    Serial.print(F(" "));

    if (Gb_Ele >= 8 && Gb_Ele <= 19) { // Weiche ?
      Serial.print('.');
    }           // Lage bits nicht auswerten
    else {
      if (Ele & FSTRSTART) Serial.print('F'); else Serial.print('-');
    }
    if (Ele & FSTRBEL  ) Serial.print('L'); else Serial.print('-');
    Serial.print(F("|"));
  }
} // Tracker()


#ifndef LOCONET
void Debug32(uint8_t valH, uint8_t valL, uint8_t val3) {
  if (GL_DebOpt & 0b00100000) { // 32
    Serial.print(F("Out H L Val: ")); Serial.print(valH); Serial.print(' '); Serial.print(valL);
    Serial.print(' '); Serial.println(val3);
  }
} // Debug32()
#else
void Debug32(uint16_t Address, uint8_t Direction) {
  if (GL_DebOpt & 0b00100000) { // 32
    Serial.print(F("Adresse: ")); Serial.print(Address); Serial.print(' '); Serial.println(Direction);
  }
} // Debug32()
#endif



uint8_t LageDW(int fx, int fy) {
  // 0 ohne Lage
  // 4 \  /  3
  // 5 ----  2
  // 6 /  \  1
  // 24     25

  uint8_t Lage = 0xFF;
  uint8_t Ele  = StpM[fy][fx];

  Lage =  Ele >> 6;
  Ele  =  Ele & 0x1F;

  switch (Ele) {
    case 24: if (Lage) Lage = Lage + 3;
      break;
    case 25:      // Lage 2 bleibt 2
      if (Lage == 3) Lage = 1;
      else if (Lage == 1) Lage = 3;
      break;
    default:
      break;
  }
  Serial.print("Ele, Lage:"); Serial.print(StpM[fy][fx] & 0x1F);
  Serial.print(' '); Serial.println(Lage);
  return Lage;
} // LageDW()




void debug (uint8_t achtbit) { // fuer Streckensymbol

  uint8_t dat = achtbit;

  dat = achtbit >> 5;
  Serial.print(F("3Bit H: 0b")); Serial.print(dat, BIN);
  dat = achtbit & 0x1F;
  Serial.print(' '); Serial.print(F(", 5Bit L: 0b")); Serial.print(dat, BIN);
  Serial.print(' '); Serial.print(F(", 5Bit L dez: ")); Serial.println(dat);

} // debug()



// Aufruf testweise mit Touch-Taste <>, gut zu gebrauchen, weil Vorschau Fweg
// hier noch über Serial Mon.
void Fliste(int16_t Fwnum) {    // Fwnum > 0
  Serial.print(F("Fweg Nr."));
  if (Fwnum < 10) Serial.print(' ');
  Serial.print(Fwnum); Serial.print(F(": "));

  for (int i = 0; Fweg[Fwnum - 1][i] != 0; i++) {
    Serial.print(Fweg[Fwnum - 1][i]); Serial.print(' ');
  }
  Serial.println();
} // Fliste()




void TheKeys(void) {
  Serial.println(F("Tasten:"));
  for (int k = 0; k < GL_ButtCnt; k++) {
    Serial.print(F("k: "));  Format3stell(k);
    Format3stell((ButtField[k].xyKo) >> 4);
    Format3stell((ButtField[k].xyKo) & 0x0F);
    Serial.print(F("Typ ")); Serial.println(ButtField[k].Keytype);
  }
  Serial.print(F("Texttaste x,y:")); Serial.print(GL_sqButtX);
  Serial.print(F(" ")); Serial.println(GL_sqButtY);
} // TheKeys()


void Format3stell(uint16_t value) {
  if (value <  10) Serial.print(F(" "));
  if (value < 100) Serial.print(F(" "));
  Serial.print(value); Serial.print(F(" "));
} // Format3stell



void CheckLinkArray(void) {

  Serial.println(F("Link-Array:"));
  for (int k = 0; k < LDIM; k++) {
    Serial.print(F("x,y  x1,y1: "));
    Serial.print(Linkxy[0][k]); Serial.print(',');
    Serial.print(Linkxy[1][k]); Serial.print(F("  "));
    Serial.print(Linkxy[2][k]); Serial.print(',');
    Serial.print(Linkxy[3][k]); Serial.println();
  }

} // CheckLinkArray()


void LinkTest(void) {
  if (LinkSearch( 7, 2)) {
    Serial.print(Lresult[0]); Serial.print(' '); Serial.print(Lresult[1]); Serial.println(' ');
  }
  else   Serial.println(F("no match"));

  if (LinkSearch(0, 4)) {
    Serial.print(Lresult[0]); Serial.print(' '); Serial.print(Lresult[1]); Serial.println(' ');
  }
  else   Serial.println(F("no match"));

  if (LinkSearch(4, 4)) {
    Serial.print(Lresult[0]); Serial.print(' '); Serial.print(Lresult[1]); Serial.println(' ');
  }
  else   Serial.println(F("no match"));

  if (LinkSearch(5, 0)) {
    Serial.print(Lresult[0]); Serial.print(' '); Serial.print(Lresult[1]); Serial.println(' ');
  }
  else   Serial.println(F("no match"));

  if (LinkSearch(0, 0)) {
    Serial.print(Lresult[0]); Serial.print(' '); Serial.print(Lresult[1]); Serial.println(' ');
  }
  else   Serial.println(F("no match"));

}



void QueDump(void) {
  uint16_t ptr = 0;
  Serial.println();
  Serial.println(F("Queue:"));
  for (uint16_t cnt = 0; cnt < XQDIM; cnt++) {
    Serial.print(XBusQueue[ptr]); ptr++; Serial.print(' ');
    Serial.print(XBusQueue[ptr]); ptr++; Serial.print(F(", "));
    Serial.print(XBusQueue[ptr]); ptr++; Serial.print(' ');
    Serial.print(XBusQueue[ptr]); ptr++;
    if (ptr == RDptr) Serial.println(F(" <"));
    else              Serial.println();
  }
} // QueDump()



// -------------------------------------------------------------------------------

// Testhilfe (Beispiel)
//Serial.print(change,HEX); Serial.print("ch,k:");
//Serial.print(k,HEX); Serial.println(' ');

// XpressNet.getTrntInfo (byte FAdr_High, byte FAdr_Low);    //Ermitteln der Schaltstellung einer Weiche

// XpressNet.setTrntPos (byte FAdr_High, byte FAdr_Low, byte Pos); //Schalten einer Weiche
