// D96.ino, Balken, mit Loknr.
// Frei-Meldungen während Init überspringen
// Queue 128 Bytes (5 x 24 = 120)

#ifndef _Arduino_h
#define _Arduino_h
#include <Arduino.h>
#endif

#ifndef _csvParser_h
#define _csvParser_h
#include "csvParser.h"
#endif

#include <stdint.h>
#include <avr/interrupt.h>

extern void Text_CR(int);
extern void AnzeigeRC(uint8_t, uint16_t, byte);
extern void AnzeigeRCLok(uint16_t, byte);
extern void TrackLine  (int fx, int fy);
extern void TrackEnviron (int fx, int fy);
extern void RCFrei(int x, int y, uint8_t Ele);
extern void RCBelegung(int x, int y, uint8_t Ele);

extern char TextBackGrnd[];
extern char TextRCBackGd[];

void    SetTxtPtr(uint8_t x, uint8_t y);
void    RCtrNumber(uint8_t Melder, uint16_t LokNum);
uint8_t Get_Frame(uint8_t);

// write/read Queue
#define RCQDIM 128
#define RCQPMSK RCQDIM-1
volatile char RCqueue[RCQDIM] = {0};
volatile uint16_t rcWRptr  =  0;
volatile uint16_t rcRDptr  =  0;

volatile uint8_t x;
volatile uint8_t d;

void RCFetch (void) {

  while (rcWRptr != rcRDptr)
  {
    d = RCqueue[rcRDptr];
    rcRDptr++;
    rcRDptr = rcRDptr & RCQPMSK;

    //if (Get_Frame(d)) return;             // eine Ausgabe pro loop()
    Get_Frame(d);
  } // while
} // RCFetch()


uint8_t Get_Frame(uint8_t dat) {

  static uint8_t  Madr = 0;
  static uint8_t  Bptr = 0;
  static uint8_t  Hadr = 0;
  uint16_t        Ladr = 0;

  switch (dat) {
    case 0xFC: Bptr = 1;
      break;
    case 0xFF: Bptr = 0;
      break;
    default:
      switch (Bptr) {
        case 1: Madr = dat;
          Bptr = 2;
          break;
        case 2: Hadr = dat & 0x7F;
          Bptr = 3;
          break;
        case 3: Ladr = (Hadr << 8) + dat;  // 0x7777 belegt, 0 clear, sonst Ladr.

          // hier Frame fertig: Madr, Ladr
          RCtrNumber(Madr, Ladr);
          Bptr = 0;
          //return 1;                  // eine Ausgabe pro loop()
          break;

        default: Bptr = 0;
          break;
      }
  }
  return 0;
} // Get_Frame()


void RCtrNumber(uint8_t Melder, uint16_t LokNum) {
  uint8_t    k = Melder - 1;
  uint8_t  xko = RMKoord[k].x;
  uint8_t  yko = RMKoord[k].y;
  uint8_t  Ele = StpM[yko][xko];
  static uint32_t FrameCnt = 0;

  FrameCnt++;

  if ((Melder < 1) | (Melder > RMDIM)) return;

  if (xko + yko) {  // Melder vorhanden

    switch (Ele & 0x1F) {  // macht Sinn
      case 1:
      case 2:
      case 3:
        switch (LokNum) {
          case 0x7777:
            RCBelegung(xko, yko, Ele);
            SetTxtPtr(xko, yko);
            AnzeigeRCLok(0, 0);  // nur roter Balken
            break;
          case 0:
            if (FrameCnt > RMDIM)  {   // wenn Init-Block, keine Freianzeige
              TrackEnviron(xko, yko);
              TrackLine(xko, yko);
              RCFrei(xko, yko, Ele);
            }
            break;
          default:
            RCBelegung(xko, yko, Ele);
            SetTxtPtr(xko, yko);
            AnzeigeRCLok(LokNum, 1);  // alte Lnr. nicht löschen
            break;
        } // switch LokNum
        break;
      default:
        break;
    } // switch Ele
  } // if Melder
} // RCtrNumber()




ISR(USART3_RX_vect)
{
  //digitalWrite(13, HIGH); //debugging code
  x = UDR3;    //receive from UART3

  RCqueue[rcWRptr] = x;
  rcWRptr++;
  rcWRptr = rcWRptr & RCQPMSK;
} // ISR()


void uart3dOut(uint8_t dat) {
  // transmitter empty? (data sheet p.191)
  while (!(UCSR3A & 0b00100000)) {}
  UDR3 = dat;
} // uart3dout()


void uart3init(void)
{
  //--configure UART3 of MEGA: Bd = 19200; RXRDY interrupt----
  UCSR3B = 0b10011000;  //
  UCSR3C = 0b00000110;  //
  //-Bd: 19200-------------
  UBRR3L = 0x33;
  UBRR3H = 0x00;
  //--global int enable--
  bitSet(SREG, 7);

} // uart3init()
